package subtitles

import (
	"fmt"
	"net/http"

	"github.com/gregdel/srt2vtt"
)

// ConvertSubtitle downloads and converts a subtitle from srt to vvt
func ConvertSubtitle(url string, w http.ResponseWriter) error {
	// Client with compression disabled
	c := &http.Client{
		Transport: &http.Transport{DisableCompression: true},
	}

	req, err := http.NewRequest("GET", url, nil)
	if err != nil {
		return err
	}

	resp, err := c.Do(req)
	if err != nil {
		return err
	}
	defer resp.Body.Close()

	if resp.StatusCode != http.StatusOK {
		return fmt.Errorf("Invalid subitle response code: %d", resp.StatusCode)
	}

	reader, err := srt2vtt.NewReader(resp.Body)
	if err != nil {
		return err
	}

	_, err = reader.WriteTo(w)
	if err != nil {
		return err
	}

	return nil
}
